import structlog

MIDDLEWARE.append(
    "django_structlog.middlewares.RequestMiddleware",
)
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        # JSONで構造化ログにするフォーマッターを定義
        "json_formatter": {
            "()": structlog.stdlib.ProcessorFormatter,
            # 日本語を読みやすくする
            "processor": (
                structlog.processors.JSONRenderer(
                  ensure_ascii=False,
                )
            ),
        },
        # 開発用にテキスト形式のフォーマッターを定義
        "plain_console": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": (
                structlog.dev.ConsoleRenderer(),
            )
        },
    },
    "handlers": {
        "debug-console": {
            "level": "DEBUG",
            "filters": ["require_debug_true"],
            "class": "logging.StreamHandler",
            "formatter": "json_formatter",
        },
        "prod-console": {
            "level": "INFO",
            "filters": ["require_debug_false"],
            "class": "logging.StreamHandler",
            "formatter": "json_formatter",
        },
    },
    "filters": {
        "require_debug_false": {
            "()": "django.utils.log.RequireDebugFalse"
        },
        "require_debug_true": {
            "()": "django.utils.log.RequireDebugTrue"
        },
    },
    "root": {
        "handlers": ["prod-console", "debug-console"],
        "level": "NOTSET",
        "propagate": False,
    },
    "loggers": {
        "django.db.backends": {
            "handlers": [
                "prod-console",
                "debug-console",
            ],
            "level": "ERROR",
            "propagate": False,
        },
        "django.request": {
            "handlers": [
                "prod-console",
                "debug-console",
            ],
            "level": "ERROR",
            "propagate": False,
        },
        "django": {
            "handlers": [
                "prod-console",
                "debug-console",
            ],
            "level": "ERROR",
            "propagate": False,
        },
    },
}
structlog.configure(
    processors=[
        # 最初の処理系としてmerge_contextvarsを追加
        structlog.contextvars.merge_contextvars,
        # ログレベルが低すぎる場合パイプラインを中断し、
        # ログエントリを破棄する
        structlog.stdlib.filter_by_level,
        # ISO 8601形式のタイムスタンプを追加する
        structlog.processors.TimeStamper(fmt="iso"),
        # イベントの辞書にロガー名を追加する
        structlog.stdlib.add_logger_name,
        # イベントの辞書にログレベルを追加する
        structlog.stdlib.add_log_level,
        # %演算子の形式でフォーマットを実行する
        structlog.stdlib.PositionalArgumentsFormatter(),
        # イベントの辞書の"stack_info"キーがTrueの場合、
        # 削除し"stack"キーに現在のスタックトレースを
        # レンダリングする
        structlog.processors.StackInfoRenderer(),
        # イベントの辞書の"exc_info"キーがTrueか
        # sys.exc_info()タプルである場合、
        # "exc_info"を削除しトレースバック付きの例外を
        # "exception"にレンダリングする
        structlog.processors.format_exc_info,
        # 値がバイトである場合、それをUnicodeのstrに
        # デコードする
        structlog.processors.UnicodeDecoder(),
        # json_formatterに指定したProcessorFormatter用に
        # 辞書の構造を変更する
        structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
    ],
    # logger_factoryは出力に使用するラップされたロガーを
    # 作成するために使用される
    # こちらはlogging.Loggerを返す
    # 最終的な処理系（JSONRenderer）からの値（JSON）は、
    # バインドしたロガーに対して呼び出したものと同じ
    # 名前のメソッドに渡される
    logger_factory=structlog.stdlib.LoggerFactory(),
    # 最初のバインドロガーを作成したあと、設定を効果的に
    # フリーズする
    cache_logger_on_first_use=True,
)
